/*
Convert  an ArrayBuffer into a string
from https://developers.google.com/web/updates/2012/06/How-to-convert-ArrayBuffer-to-and-from-String
*/
function ab2str(buf) {
    return String.fromCharCode.apply(null, new Uint8Array(buf));
}

function exportedKeyToPem(key, type) {
    let exportedAsString = ab2str(key);
    let exportedAsBase64 = window.btoa(exportedAsString);
    return `-----BEGIN ${type.toUpperCase()} KEY-----\n${exportedAsBase64}\n-----END ${type.toUpperCase()} KEY-----`;
}

async function genKey() {
  // Generate keys
    const { publicKey, privateKey } = await crypto.subtle.generateKey(
      // {
      //   name: "ECDSA",
      //   namedCurve: "P-384",
      // },
      {
        name: "Ed25519",
      },
      true,
      ["sign", "verify"],
    );
    const exportedPubkey = await crypto.subtle.exportKey("spki", publicKey);
    const exportedPrivkey = await crypto.subtle.exportKey("pkcs8", privateKey);
    // const privkey = await crypto.subtle.wrapKey("pkcs8", privateKey, wrapkey, { name: "AES-KW" });

    console.log(`publicKey: ${exportedKeyToPem(exportedPubkey, publicKey.type)}`);
    console.log(`privateKey: ${exportedKeyToPem(exportedPrivkey, privateKey.type)}`);
}

async function test(data) {
  console.log(`Message: ${data}`);
  try {
    // Generate keys
    const { publicKey, privateKey } = await crypto.subtle.generateKey(
      {
        name: "Ed25519",
      },
      true,
      ["sign", "verify"],
    );

    console.log(`publicKey: ${publicKey}, type: ${publicKey.type}`);
    console.log(`privateKey: ${privateKey},  type: ${privateKey.type}`);

    // Encode data prior to signing
    const encoder = new TextEncoder();
    encodedData = encoder.encode(data);

    // Log the first part of the encoded data
    const shorterEncodedBuffer = new Uint8Array(encodedData.buffer, 0, 14);
    console.log(
      `encodedData: ${shorterEncodedBuffer}...[${encodedData.byteLength} bytes total]`,
    );
    //console.log(`encodedData: ${encodedData}`);

    // Sign the data using the private key.
    const signature = await crypto.subtle.sign(
      {
        name: "Ed25519",
      },
      privateKey,
      encodedData,
    );

    // Log the first part of the signature data
    const signatureBuffer = new Uint8Array(signature, 0, 14);
    console.log(
      `signature: ${signatureBuffer}...[${signature.byteLength} bytes total]`,
    );

    // Verify the signature using the public key
    const verifyResult = await crypto.subtle.verify(
      {
        name: "Ed25519",
      },
      publicKey,
      signature,
      encodedData,
    );

    // Log result - true if the text was signed with the corresponding public key.
    console.log(`signature verified?: ${verifyResult}`);
  } catch (error) {
    console.log(error);
  }
}

test('ceci est un test');
genKey()
